import { useState, useEffect } from 'react';
import debounce from 'lodash/debounce';

var isScrollable = function (element) {
  return element.scrollHeight > element.clientHeight;
};

var scrollHandler = function (onScroll) {
  return function (event) {
    if (!(event.target instanceof HTMLElement)) return;
    var containerHeight = event.target.clientHeight;
    var contentHeight = event.target.scrollHeight;
    var contentScrollY = event.target.scrollTop;
    var percentage = contentScrollY / (contentHeight - containerHeight) * 100;
    onScroll(Math.round(percentage));
  };
};

var useScrollObserver = function (onScroll) {
  var _a = useState(null),
      element = _a[0],
      setElement = _a[1];

  useEffect(function () {
    if (!element) return;
    if (!isScrollable(element)) return;
    var debouncedHandler = debounce(scrollHandler(onScroll), 50, {
      leading: true,
      trailing: true
    });
    element.addEventListener('scroll', debouncedHandler);
    element.dispatchEvent(new Event('scroll')); // eslint-disable-next-line consistent-return

    return function () {
      return element.removeEventListener('scroll', debouncedHandler);
    };
  }, [element, onScroll]);
  return {
    setElement: setElement
  };
};

export { useScrollObserver as default };
//# sourceMappingURL=useScrollObserver.js.map
